@@ -108,15 +108,15 @@ real_active (NautilusWindowSlot *slot)
 {
 	NautilusWindow *window;
 
-	window = slot->window;
+	window = slot->pane->window;
 
 	/* sync window to new slot */
 	nautilus_window_sync_status (window);
 	nautilus_window_sync_allow_stop (window, slot);
 	nautilus_window_sync_title (window, slot);
 	nautilus_window_sync_zoom_widgets (window);
-	nautilus_window_sync_location_widgets (window);
-	nautilus_window_sync_search_widgets (window);
+	nautilus_window_pane_sync_location_widgets (slot->pane);
+	nautilus_window_pane_sync_search_widgets (slot->pane);
 
 	if (slot->viewed_file != NULL) {
 		nautilus_window_load_view_as_menus (window);
@@ -128,12 +128,14 @@ static void
 nautilus_window_slot_active (NautilusWindowSlot *slot)
 {
 	NautilusWindow *window;
+	NautilusWindowPane *pane;
 
 	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));
 
-	window = NAUTILUS_WINDOW (slot->window);
-	g_assert (g_list_find (window->details->slots, slot) != NULL);
-	g_assert (slot == window->details->active_slot);
+	pane = NAUTILUS_WINDOW_PANE (slot->pane);
+	window = NAUTILUS_WINDOW (slot->pane->window);
+	g_assert (g_list_find (pane->slots, slot) != NULL);
+	g_assert (slot == window->details->active_pane->active_slot);
 
 	EEL_CALL_METHOD (NAUTILUS_WINDOW_SLOT_CLASS, slot,
 			 active, (slot));
@@ -144,20 +146,23 @@ real_inactive (NautilusWindowSlot *slot)
 {
 	NautilusWindow *window;
 
-	window = NAUTILUS_WINDOW (slot->window);
-	g_assert (slot == window->details->active_slot);
+	window = NAUTILUS_WINDOW (slot->pane->window);
+	g_assert (slot == window->details->active_pane->active_slot);
 }
 
 static void
 nautilus_window_slot_inactive (NautilusWindowSlot *slot)
 {
 	NautilusWindow *window;
+	NautilusWindowPane *pane;
 
 	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));
 
-	window = NAUTILUS_WINDOW (slot->window);
-	g_assert (g_list_find (window->details->slots, slot) != NULL);
-	g_assert (slot == window->details->active_slot);
+	pane = NAUTILUS_WINDOW_PANE (slot->pane);
+	window = NAUTILUS_WINDOW (pane->window);
+
+	g_assert (g_list_find (pane->slots, slot) != NULL);
+	g_assert (slot == window->details->active_pane->active_slot);
 
 	EEL_CALL_METHOD (NAUTILUS_WINDOW_SLOT_CLASS, slot,
 			 inactive, (slot));
@@ -167,26 +172,35 @@ nautilus_window_slot_inactive (NautilusWindowSlot *slot)
 static void
 nautilus_window_slot_init (NautilusWindowSlot *slot)
 {
-	GtkWidget *content_box, *eventbox, *extras_vbox;
+	GtkWidget *content_box, *eventbox, *extras_vbox, *frame, *hsep;
 
 	content_box = gtk_vbox_new (FALSE, 0);
 	slot->content_box = content_box;
 	gtk_widget_show (content_box);
 
+	frame = gtk_frame_new (NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
+	gtk_box_pack_start (GTK_BOX (content_box), frame, TRUE, TRUE, 0);
+	gtk_widget_show (frame);
+
+	slot->view_box = gtk_vbox_new (FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (frame), slot->view_box);
+	gtk_widget_show (slot->view_box);
+
 	eventbox = gtk_event_box_new ();
 	slot->extra_location_event_box = eventbox;
 	gtk_widget_set_name (eventbox, "nautilus-extra-view-widget");
-	gtk_box_pack_start (GTK_BOX (content_box), eventbox, FALSE, FALSE, 0);
-	
+	gtk_box_pack_start (GTK_BOX (slot->view_box), eventbox, FALSE, FALSE, 0);
+
 	extras_vbox = gtk_vbox_new (FALSE, 6);
 	gtk_container_set_border_width (GTK_CONTAINER (extras_vbox), 6);
 	slot->extra_location_widgets = extras_vbox;
 	gtk_container_add (GTK_CONTAINER (eventbox), extras_vbox);
 	gtk_widget_show (extras_vbox);
 
-	slot->view_box = gtk_vbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (content_box), slot->view_box, TRUE, TRUE, 0);
-	gtk_widget_show (slot->view_box);
+	hsep = gtk_hseparator_new ();
+	gtk_box_pack_start (GTK_BOX (slot->view_box), hsep, FALSE, FALSE, 0);
+	slot->extra_location_separator = hsep;
 
 	slot->title = g_strdup (_("Loading..."));
 }
@@ -216,7 +230,7 @@ GFile *
 nautilus_window_slot_get_location (NautilusWindowSlot *slot)
 {
 	g_assert (slot != NULL);
-	g_assert (NAUTILUS_IS_WINDOW (slot->window));
+	g_assert (NAUTILUS_IS_WINDOW (slot->pane->window));
 
 	if (slot->location != NULL) {
 		return g_object_ref (slot->location);
@@ -235,6 +249,15 @@ nautilus_window_slot_get_location_uri (NautilusWindowSlotInfo *slot)
 	return NULL;
 }
 
+static void
+nautilus_window_slot_make_hosting_pane_active (NautilusWindowSlot *slot)
+{
+	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));
+	g_assert (NAUTILUS_IS_WINDOW_PANE (slot->pane));
+	
+	nautilus_window_set_active_slot (slot->pane->window, slot);
+}
+
 char *
 nautilus_window_slot_get_title (NautilusWindowSlot *slot)
 {
@@ -260,7 +283,7 @@ static NautilusWindow *
 nautilus_window_slot_get_window (NautilusWindowSlot *slot)
 {
 	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));
-	return slot->window;
+	return slot->pane->window;
 }
 
 /* nautilus_window_slot_set_title:
@@ -278,7 +301,7 @@ nautilus_window_slot_set_title (NautilusWindowSlot *slot,
 
 	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));
 
-	window = NAUTILUS_WINDOW (slot->window);
+	window = NAUTILUS_WINDOW (slot->pane->window);
 
 	changed = FALSE;
 
@@ -335,7 +358,7 @@ nautilus_window_slot_update_icon (NautilusWindowSlot *slot)
 	const char *icon_name;
 	GdkPixbuf *pixbuf;
 
-	window = slot->window;
+	window = slot->pane->window;
 
 	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 
@@ -366,28 +389,32 @@ nautilus_window_slot_update_icon (NautilusWindowSlot *slot)
 	}
 }
 
-static void
-title_changed_callback (NautilusView *view,
-			NautilusWindowSlot *slot)
+void
+nautilus_window_slot_is_in_active_pane (NautilusWindowSlot *slot,
+					gboolean is_active)
 {
-        g_assert (NAUTILUS_IS_WINDOW (slot->window));
+	/* NULL is valid, and happens during init */
+	if (!slot) {
+		return;
+	}
 
-        nautilus_window_slot_update_title (slot);
-	nautilus_window_slot_update_icon (slot);
+	/* it may also be that the content is not a valid directory view during init */
+	if (slot->content_view != NULL) {
+		nautilus_view_set_is_active (slot->content_view, is_active);
+	}
+
+	if (slot->new_content_view != NULL) {
+		nautilus_view_set_is_active (slot->new_content_view, is_active);
+	}
 }
 
-
 void
 nautilus_window_slot_connect_content_view (NautilusWindowSlot *slot,
 					   NautilusView *view)
 {
 	NautilusWindow *window;
 
-	g_signal_connect (view, "title-changed",
-			  G_CALLBACK (title_changed_callback),
-			  slot);
-
-	window = slot->window;
+	window = slot->pane->window;
 	if (window != NULL && slot == nautilus_window_get_active_slot (window)) {
 		nautilus_window_connect_content_view (window, view);
 	}
@@ -399,10 +426,8 @@ nautilus_window_slot_disconnect_content_view (NautilusWindowSlot *slot,
 {
 	NautilusWindow *window;
 
-	g_signal_handlers_disconnect_by_func (view, G_CALLBACK (title_changed_callback), slot);
-
-	window = slot->window;
-	if (window != NULL && slot == nautilus_window_get_active_slot (window)) {
+	window = slot->pane->window;
+	if (window != NULL && window->details->active_pane && window->details->active_pane->active_slot == slot) {
 		nautilus_window_disconnect_content_view (window, view);
 	}
 }
@@ -414,7 +439,7 @@ nautilus_window_slot_set_content_view_widget (NautilusWindowSlot *slot,
 	NautilusWindow *window;
 	GtkWidget *widget;
 
-	window = slot->window;
+	window = slot->pane->window;
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
 	if (slot->content_view != NULL) {
@@ -452,7 +477,7 @@ nautilus_window_slot_set_allow_stop (NautilusWindowSlot *slot,
 
 	slot->allow_stop = allow;
 
-	window = NAUTILUS_WINDOW (slot->window);
+	window = NAUTILUS_WINDOW (slot->pane->window);
 	nautilus_window_sync_allow_stop (window, slot);
 }
 
@@ -467,8 +492,8 @@ nautilus_window_slot_set_status (NautilusWindowSlot *slot,
 	g_free (slot->status_text);
 	slot->status_text = g_strdup (status);
 
-	window = NAUTILUS_WINDOW (slot->window);
-	if (slot == window->details->active_slot) {
+	window = NAUTILUS_WINDOW (slot->pane->window);
+	if (slot == window->details->active_pane->active_slot) {
 		nautilus_window_sync_status (window);
 	}
 }
@@ -511,6 +536,7 @@ nautilus_window_slot_remove_extra_location_widgets (NautilusWindowSlot *slot)
 			       remove_all,
 			       slot->extra_location_widgets);
 	gtk_widget_hide (slot->extra_location_event_box);
+	gtk_widget_hide (slot->extra_location_separator);
 }
 
 void
@@ -520,13 +546,14 @@ nautilus_window_slot_add_extra_location_widget (NautilusWindowSlot *slot,
 	gtk_box_pack_start (GTK_BOX (slot->extra_location_widgets),
 			    widget, TRUE, TRUE, 0);
 	gtk_widget_show (slot->extra_location_event_box);
+	gtk_widget_show (slot->extra_location_separator);
 }
 
 void
 nautilus_window_slot_add_current_location_to_history_list (NautilusWindowSlot *slot)
 {
 
-	if ((slot->window == NULL || !NAUTILUS_IS_DESKTOP_WINDOW (slot->window)) &&
+	if ((slot->pane->window == NULL || !NAUTILUS_IS_DESKTOP_WINDOW (slot->pane->window)) &&
 	    nautilus_add_bookmark_to_history_list (slot->current_location_bookmark)) {
 		nautilus_send_history_list_changed ();
 	}
@@ -618,7 +645,7 @@ nautilus_window_slot_dispose (GObject *object)
 		slot->find_mount_cancellable = NULL;
 	}
 
-	slot->window = NULL;
+	slot->pane = NULL;
 
 	g_free (slot->title);
 	slot->title = NULL;
@@ -641,5 +668,6 @@ nautilus_window_slot_info_iface_init (NautilusWindowSlotInfoIface *iface)
 	iface->set_status = nautilus_window_slot_set_status;
 	iface->get_title = nautilus_window_slot_get_title;
 	iface->open_location = nautilus_window_slot_open_location_full;
+	iface->make_hosting_pane_active = nautilus_window_slot_make_hosting_pane_active; 
 }
 
